# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Union, Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["PlanGroupResponse"]


class PlanGroupResponse(BaseModel):
    id: str
    """The UUID of the entity."""

    version: int
    """The version number:

    - **Create:** On initial Create to insert a new entity, the version is set at 1
      in the response.
    - **Update:** On successful Update, the version is incremented by 1 in the
      response.
    """

    account_id: Optional[str] = FieldInfo(alias="accountId", default=None)
    """Optional.

    This PlanGroup was created as bespoke for the associated Account with this
    Account ID.
    """

    code: Optional[str] = None
    """The short code representing the PlanGroup."""

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """The unique identifier (UUID) for the user who created the PlanGroup."""

    currency: Optional[str] = None
    """Currency code for the PlanGroup (For example, USD)."""

    custom_fields: Optional[Dict[str, Union[str, float]]] = FieldInfo(alias="customFields", default=None)
    """User defined fields enabling you to attach custom data.

    The value for a custom field can be either a string or a number.

    If `customFields` can also be defined for this entity at the Organizational
    level,`customField` values defined at individual level override values of
    `customFields` with the same name defined at Organization level.

    See
    [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
    in the m3ter documentation for more information.
    """

    dt_created: Optional[datetime] = FieldInfo(alias="dtCreated", default=None)
    """The date and time _(in ISO 8601 format)_ when the PlanGroup was first created."""

    dt_last_modified: Optional[datetime] = FieldInfo(alias="dtLastModified", default=None)
    """The date and time _(in ISO 8601 format)_ when the PlanGroup was last modified."""

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """The unique identifier (UUID) for the user who last modified the PlanGroup."""

    minimum_spend: Optional[float] = FieldInfo(alias="minimumSpend", default=None)
    """The minimum spend amount for the PlanGroup."""

    minimum_spend_accounting_product_id: Optional[str] = FieldInfo(
        alias="minimumSpendAccountingProductId", default=None
    )
    """Optional.

    Product ID to attribute the PlanGroup's minimum spend for accounting purposes.
    """

    minimum_spend_bill_in_advance: Optional[bool] = FieldInfo(alias="minimumSpendBillInAdvance", default=None)
    """A boolean flag that determines when the minimum spend is billed.

    This flag overrides the setting at Organizational level for minimum spend
    billing in arrears/in advance.

    - **TRUE** - minimum spend is billed at the start of each billing period.
    - **FALSE** - minimum spend is billed at the end of each billing period.
    """

    minimum_spend_description: Optional[str] = FieldInfo(alias="minimumSpendDescription", default=None)
    """Description of the minimum spend, displayed on the bill line item."""

    name: Optional[str] = None
    """The name of the PlanGroup."""

    standing_charge: Optional[float] = FieldInfo(alias="standingCharge", default=None)
    """Standing charge amount for the PlanGroup."""

    standing_charge_accounting_product_id: Optional[str] = FieldInfo(
        alias="standingChargeAccountingProductId", default=None
    )
    """Optional.

    Product ID to attribute the PlanGroup's standing charge for accounting purposes.
    """

    standing_charge_bill_in_advance: Optional[bool] = FieldInfo(alias="standingChargeBillInAdvance", default=None)
    """A boolean flag that determines when the standing charge is billed.

    This flag overrides the setting at Organizational level for standing charge
    billing in arrears/in advance.

    - **TRUE** - standing charge is billed at the start of each billing period.
    - **FALSE** - standing charge is billed at the end of each billing period.
    """

    standing_charge_description: Optional[str] = FieldInfo(alias="standingChargeDescription", default=None)
    """Description of the standing charge, displayed on the bill line item."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union
from datetime import date, datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["AccountPlanCreateParams"]


class AccountPlanCreateParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    account_id: Required[Annotated[str, PropertyInfo(alias="accountId")]]
    """The unique identifier (UUID) for the Account."""

    start_date: Required[Annotated[Union[str, datetime], PropertyInfo(alias="startDate", format="iso8601")]]
    """
    The start date _(in ISO-8601 format)_ for the AccountPlan or AccountPlanGroup
    becoming active for the Account.
    """

    bill_epoch: Annotated[Union[str, date], PropertyInfo(alias="billEpoch", format="iso8601")]
    """
    Optional setting to define a _billing cycle date_, which acts as a reference for
    when in the applied billing frequency period bills are created:

    - For example, if you attach a Plan to an Account where the Plan is configured
      for monthly billing frequency and you've defined the period the Plan will
      apply to the Account to be from January 1st, 2022 until January 1st, 2023. You
      then set a `billEpoch` date of February 15th, 2022. The first Bill will be
      created for the Account on February 15th, and subsequent Bills created on the
      15th of the months following for the remainder of the billing period - March
      15th, April 15th, and so on.
    - If not defined, then the `billEpoch` date set for the Account will be used
      instead.
    - The date is in ISO-8601 format.
    """

    child_billing_mode: Annotated[
        Literal["PARENT_SUMMARY", "PARENT_BREAKDOWN", "CHILD"], PropertyInfo(alias="childBillingMode")
    ]
    """
    If the Account is either a Parent or a Child Account, this specifies the Account
    hierarchy billing mode. The mode determines how billing will be handled and
    shown on bills for charges due on the Parent Account, and charges due on Child
    Accounts:

    - **Parent Breakdown** - a separate bill line item per Account. Default setting.

    - **Parent Summary** - single bill line item for all Accounts.

    - **Child** - the Child Account is billed.
    """

    code: str
    """A unique short code for the AccountPlan or AccountPlanGroup."""

    contract_id: Annotated[str, PropertyInfo(alias="contractId")]
    """
    The unique identifier (UUID) for a Contract to which you want to add the Plan or
    Plan Group being attached to the Account.
    """

    custom_fields: Annotated[Dict[str, Union[str, float]], PropertyInfo(alias="customFields")]
    """User defined fields enabling you to attach custom data.

    The value for a custom field can be either a string or a number.

    If `customFields` can also be defined for this entity at the Organizational
    level, `customField` values defined at individual level override values of
    `customFields` with the same name defined at Organization level.

    See
    [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
    in the m3ter documentation for more information.
    """

    end_date: Annotated[Union[str, datetime], PropertyInfo(alias="endDate", format="iso8601")]
    """
    The end date _(in ISO-8601 format)_ for when the AccountPlan or AccountPlanGroup
    ceases to be active for the Account. If not specified, the AccountPlan or
    AccountPlanGroup remains active indefinitely.
    """

    plan_group_id: Annotated[str, PropertyInfo(alias="planGroupId")]
    """
    The unique identifier (UUID) of the PlanGroup to be attached to the Account to
    create an AccountPlanGroup.

    **Note:** Exclusive of the `planId` request parameter - exactly one of `planId`
    or `planGroupId` must be used per call.
    """

    plan_id: Annotated[str, PropertyInfo(alias="planId")]
    """
    The unique identifier (UUID) of the Plan to be attached to the Account to create
    an AccountPlan.

    **Note:** Exclusive of the `planGroupId` request parameter - exactly one of
    `planId` or `planGroupId` must be used per call.
    """

    version: int
    """The version number of the entity:

    - **Create entity:** Not valid for initial insertion of new entity - _do not use
      for Create_. On initial Create, version is set at 1 and listed in the
      response.
    - **Update Entity:** On Update, version is required and must match the existing
      version because a check is performed to ensure sequential versioning is
      preserved. Version is incremented by 1 and listed in the response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Optional
from datetime import date, datetime
from typing_extensions import Literal

import httpx

from ..types import (
    account_list_params,
    account_create_params,
    account_search_params,
    account_update_params,
    account_get_children_params,
    account_end_date_billing_entities_params,
)
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursor, AsyncCursor
from .._base_client import AsyncPaginator, make_request_options
from ..types.address_param import AddressParam
from ..types.account_response import AccountResponse
from ..types.account_search_response import AccountSearchResponse
from ..types.account_end_date_billing_entities_response import AccountEndDateBillingEntitiesResponse

__all__ = ["AccountsResource", "AsyncAccountsResource"]


class AccountsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AccountsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AccountsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AccountsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AccountsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        org_id: str | None = None,
        code: str,
        email_address: str,
        name: str,
        address: AddressParam | NotGiven = NOT_GIVEN,
        auto_generate_statement_mode: Literal["NONE", "JSON", "JSON_AND_CSV"] | NotGiven = NOT_GIVEN,
        bill_epoch: Union[str, date] | NotGiven = NOT_GIVEN,
        config_data: Dict[str, object] | NotGiven = NOT_GIVEN,
        credit_application_order: List[Literal["PREPAYMENT", "BALANCE"]] | NotGiven = NOT_GIVEN,
        currency: str | NotGiven = NOT_GIVEN,
        custom_fields: Dict[str, Union[str, float]] | NotGiven = NOT_GIVEN,
        days_before_bill_due: int | NotGiven = NOT_GIVEN,
        parent_account_id: str | NotGiven = NOT_GIVEN,
        purchase_order_number: str | NotGiven = NOT_GIVEN,
        statement_definition_id: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AccountResponse:
        """Create a new Account within the Organization.

        Args:
          code: Code of the Account.

        This is a unique short code used for the Account.

          email_address: Contact email for the Account.

          name: Name of the Account.

          address: Contact address.

          auto_generate_statement_mode: Specify whether to auto-generate statements once Bills are approved or locked.

              - **None**. Statements will not be auto-generated.
              - **JSON**. Statements are auto-generated in JSON format.
              - **JSON and CSV**. Statements are auto-generated in both JSON and CSV formats.

          bill_epoch: Optional setting to define a _billing cycle date_, which sets the date of the
              first Bill and acts as a reference for when in the applied billing frequency
              period subsequent bills are created:

              - For example, if you attach a Plan to an Account where the Plan is configured
                for monthly billing frequency and you've defined the period the Plan will
                apply to the Account to be from January 1st, 2022 until January 1st, 2023. You
                then set a `billEpoch` date of February 15th, 2022. The first Bill will be
                created for the Account on February 15th, and subsequent Bills created on the
                15th of the months following for the remainder of the billing period - March
                15th, April 15th, and so on.
              - If not defined, then the relevant Epoch date set for the billing frequency
                period at Organization level will be used instead.
              - The date is in ISO-8601 format.

          config_data:
              Configuration data for the Account Supported settings:

              - SendBillsToThirdParties ("true"/"false")

          credit_application_order: Define the order in which any Prepayment or Balance amounts on the Account are
              to be drawn-down against for billing. Four options:

              - `"PREPAYMENT","BALANCE"`. Draw-down against Prepayment credit before Balance
                credit.
              - `"BALANCE","PREPAYMENT"`. Draw-down against Balance credit before Prepayment
                credit.
              - `"PREPAYMENT"`. Only draw-down against Prepayment credit.
              - `"BALANCE"`. Only draw-down against Balance credit.

              **NOTES:**

              - Any setting you define here overrides the setting for credit application order
                at Organization level.
              - If the Account belongs to a Parent/Child Account hierarchy, then the
                `creditApplicationOrder` settings are not available, and the draw-down order
                defaults always to Prepayment then Balance order.

          currency:
              Account level billing currency, such as USD or GBP. Optional attribute:

              - If you define an Account currency, this will be used for bills.
              - If you do not define a currency, the billing currency defined at
                Organizational level will be used.

              **Note:** If you've attached a Plan to the Account that uses a different
              currency to the billing currency, then you must add the relevant currency
              conversion rate at Organization level to ensure the billing process can convert
              line items calculated using the Plan currency into the selected billing
              currency. If you don't add these conversion rates, then bills will fail for the
              Account.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          days_before_bill_due: Enter the number of days after the Bill generation date that you want to show on
              Bills as the due date.

              **Note:** If you define `daysBeforeBillDue` at individual Account level, this
              will take precedence over any `daysBeforeBillDue` setting defined at
              Organization level.

          parent_account_id: Parent Account ID, or null if this Account does not have a parent.

          purchase_order_number: Purchase Order Number of the Account.

              Optional attribute - allows you to set a purchase order number that comes
              through into invoicing. For example, your financial systems might require this
              as a reference for clearing payments.

          statement_definition_id: The UUID of the statement definition used when Bill statements are generated for
              the Account. If no statement definition is specified for the Account, the
              statement definition specified at Organizational level is used.

              Bill statements can be used as informative backing sheets to invoices. Based on
              the usage breakdown defined in the statement definition, generated statements
              give a breakdown of usage charges on Account Bills, which helps customers better
              understand usage charges incurred over the billing period.

              See
              [Working with Bill Statements](https://www.m3ter.com/docs/guides/running-viewing-and-managing-bills/working-with-bill-statements)
              in the m3ter documentation for more details.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._post(
            f"/organizations/{org_id}/accounts",
            body=maybe_transform(
                {
                    "code": code,
                    "email_address": email_address,
                    "name": name,
                    "address": address,
                    "auto_generate_statement_mode": auto_generate_statement_mode,
                    "bill_epoch": bill_epoch,
                    "config_data": config_data,
                    "credit_application_order": credit_application_order,
                    "currency": currency,
                    "custom_fields": custom_fields,
                    "days_before_bill_due": days_before_bill_due,
                    "parent_account_id": parent_account_id,
                    "purchase_order_number": purchase_order_number,
                    "statement_definition_id": statement_definition_id,
                    "version": version,
                },
                account_create_params.AccountCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AccountResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AccountResponse:
        """
        Retrieve the Account with the given Account UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/accounts/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AccountResponse,
        )

    def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        code: str,
        email_address: str,
        name: str,
        address: AddressParam | NotGiven = NOT_GIVEN,
        auto_generate_statement_mode: Literal["NONE", "JSON", "JSON_AND_CSV"] | NotGiven = NOT_GIVEN,
        bill_epoch: Union[str, date] | NotGiven = NOT_GIVEN,
        config_data: Dict[str, object] | NotGiven = NOT_GIVEN,
        credit_application_order: List[Literal["PREPAYMENT", "BALANCE"]] | NotGiven = NOT_GIVEN,
        currency: str | NotGiven = NOT_GIVEN,
        custom_fields: Dict[str, Union[str, float]] | NotGiven = NOT_GIVEN,
        days_before_bill_due: int | NotGiven = NOT_GIVEN,
        parent_account_id: str | NotGiven = NOT_GIVEN,
        purchase_order_number: str | NotGiven = NOT_GIVEN,
        statement_definition_id: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AccountResponse:
        """
        Update the Account with the given Account UUID.

        **Note:** If you have created Custom Fields for an Account, when you use this
        endpoint to update the Account, use the `customFields` parameter to preserve
        those Custom Fields. If you omit them from the update request, they will be
        lost.

        Args:
          code: Code of the Account. This is a unique short code used for the Account.

          email_address: Contact email for the Account.

          name: Name of the Account.

          address: Contact address.

          auto_generate_statement_mode: Specify whether to auto-generate statements once Bills are approved or locked.

              - **None**. Statements will not be auto-generated.
              - **JSON**. Statements are auto-generated in JSON format.
              - **JSON and CSV**. Statements are auto-generated in both JSON and CSV formats.

          bill_epoch: Optional setting to define a _billing cycle date_, which sets the date of the
              first Bill and acts as a reference for when in the applied billing frequency
              period subsequent bills are created:

              - For example, if you attach a Plan to an Account where the Plan is configured
                for monthly billing frequency and you've defined the period the Plan will
                apply to the Account to be from January 1st, 2022 until January 1st, 2023. You
                then set a `billEpoch` date of February 15th, 2022. The first Bill will be
                created for the Account on February 15th, and subsequent Bills created on the
                15th of the months following for the remainder of the billing period - March
                15th, April 15th, and so on.
              - If not defined, then the relevant Epoch date set for the billing frequency
                period at Organization level will be used instead.
              - The date is in ISO-8601 format.

          config_data:
              Configuration data for the Account Supported settings:

              - SendBillsToThirdParties ("true"/"false")

          credit_application_order: Define the order in which any Prepayment or Balance amounts on the Account are
              to be drawn-down against for billing. Four options:

              - `"PREPAYMENT","BALANCE"`. Draw-down against Prepayment credit before Balance
                credit.
              - `"BALANCE","PREPAYMENT"`. Draw-down against Balance credit before Prepayment
                credit.
              - `"PREPAYMENT"`. Only draw-down against Prepayment credit.
              - `"BALANCE"`. Only draw-down against Balance credit.

              **NOTES:**

              - Any setting you define here overrides the setting for credit application order
                at Organization level.
              - If the Account belongs to a Parent/Child Account hierarchy, then the
                `creditApplicationOrder` settings are not available, and the draw-down order
                defaults always to Prepayment then Balance order.

          currency:
              Account level billing currency, such as USD or GBP. Optional attribute:

              - If you define an Account currency, this will be used for bills.
              - If you do not define a currency, the billing currency defined at
                Organizational level will be used.

              **Note:** If you've attached a Plan to the Account that uses a different
              currency to the billing currency, then you must add the relevant currency
              conversion rate at Organization level to ensure the billing process can convert
              line items calculated using the Plan currency into the selected billing
              currency. If you don't add these conversion rates, then bills will fail for the
              Account.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          days_before_bill_due: Enter the number of days after the Bill generation date that you want to show on
              Bills as the due date.

              **Note:** If you define `daysBeforeBillDue` at individual Account level, this
              will take precedence over any `daysBeforeBillDue` setting defined at
              Organization level.

          parent_account_id: Parent Account ID, or null if this Account does not have a parent.

          purchase_order_number: Purchase Order Number of the Account.

              Optional attribute - allows you to set a purchase order number that comes
              through into invoicing. For example, your financial systems might require this
              as a reference for clearing payments.

          statement_definition_id: The UUID of the statement definition used when Bill statements are generated for
              the Account. If no statement definition is specified for the Account, the
              statement definition specified at Organizational level is used.

              Bill statements can be used as informative backing sheets to invoices. Based on
              the usage breakdown defined in the statement definition, generated statements
              give a breakdown of usage charges on Account Bills, which helps customers better
              understand usage charges incurred over the billing period.

              See
              [Working with Bill Statements](https://www.m3ter.com/docs/guides/running-viewing-and-managing-bills/working-with-bill-statements)
              in the m3ter documentation for more details.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/organizations/{org_id}/accounts/{id}",
            body=maybe_transform(
                {
                    "code": code,
                    "email_address": email_address,
                    "name": name,
                    "address": address,
                    "auto_generate_statement_mode": auto_generate_statement_mode,
                    "bill_epoch": bill_epoch,
                    "config_data": config_data,
                    "credit_application_order": credit_application_order,
                    "currency": currency,
                    "custom_fields": custom_fields,
                    "days_before_bill_due": days_before_bill_due,
                    "parent_account_id": parent_account_id,
                    "purchase_order_number": purchase_order_number,
                    "statement_definition_id": statement_definition_id,
                    "version": version,
                },
                account_update_params.AccountUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AccountResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        codes: List[str] | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursor[AccountResponse]:
        """
        Retrieve a list of Accounts that can be filtered by Account ID or Account Code.

        Args:
          codes: List of Account Codes to retrieve. These are unique short codes for each
              Account.

          ids: List of Account IDs to retrieve.

          next_token: `nextToken` for multi-page retrievals.

          page_size: Number of accounts to retrieve per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/accounts",
            page=SyncCursor[AccountResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "codes": codes,
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    account_list_params.AccountListParams,
                ),
            ),
            model=AccountResponse,
        )

    def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AccountResponse:
        """Delete the Account with the given UUID.

        This may fail if there are any
        AccountPlans that reference the Account being deleted.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/organizations/{org_id}/accounts/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AccountResponse,
        )

    def end_date_billing_entities(
        self,
        id: str,
        *,
        org_id: str | None = None,
        billing_entities: List[Literal["CONTRACT", "ACCOUNTPLAN", "PREPAYMENT", "PRICINGS", "COUNTER_PRICINGS"]],
        end_date: Union[str, datetime],
        apply_to_children: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AccountEndDateBillingEntitiesResponse:
        """
        Apply the specified end-date to billing entities associated with an Account.

        **NOTE:**

        - When you successfully end-date billing entities, the version number of each
          entity is incremented.

        Args:
          billing_entities: Defines which billing entities associated with the Account will have the
              specified end-date applied. For example, if you want the specified end-date to
              be applied to all Prepayments/Commitments created for the Account use
              `"PREPAYMENT"`.

          end_date: The end date and time applied to the specified billing entities _(in ISO 8601
              format)_.

          apply_to_children: A Boolean TRUE/FALSE flag. For Parent Accounts, set to TRUE if you want the
              specified end-date to be applied to any billing entities associated with Child
              Accounts. _(Optional)_

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/organizations/{org_id}/accounts/{id}/enddatebillingentities",
            body=maybe_transform(
                {
                    "billing_entities": billing_entities,
                    "end_date": end_date,
                    "apply_to_children": apply_to_children,
                },
                account_end_date_billing_entities_params.AccountEndDateBillingEntitiesParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AccountEndDateBillingEntitiesResponse,
        )

    def get_children(
        self,
        id: str,
        *,
        org_id: str | None = None,
        next_token: Optional[str] | NotGiven = NOT_GIVEN,
        page_size: Optional[int] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AccountResponse:
        """
        Retrieve a list of Accounts that are children of the specified Account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/accounts/{id}/children",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    account_get_children_params.AccountGetChildrenParams,
                ),
            ),
            cast_to=AccountResponse,
        )

    def search(
        self,
        *,
        org_id: str | None = None,
        from_document: int | NotGiven = NOT_GIVEN,
        operator: Literal["AND", "OR"] | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        search_query: str | NotGiven = NOT_GIVEN,
        sort_by: str | NotGiven = NOT_GIVEN,
        sort_order: Literal["ASC", "DESC"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AccountSearchResponse:
        """
        Search for Account entities.

        This endpoint executes a search query for Accounts based on the user specified
        search criteria. The search query is customizable, allowing for complex nested
        conditions and sorting. The returned list of Accounts can be paginated for
        easier management.

        Args:
          from_document: `fromDocument` for multi page retrievals.

          operator: Search Operator to be used while querying search.

          page_size: Number of Accounts to retrieve per page.

              **NOTE:** If not defined, default is 10.

          search_query:
              Query for data using special syntax:

              - Query parameters should be delimited using the $ (dollar sign).
              - Allowed comparators are:
                - (greater than) >
                - (greater than or equal to) >=
                - (equal to) :
                - (less than) <
                - (less than or equal to) <=
                - (match phrase/prefix) ~
              - Allowed parameters are: name, code, currency, purchaseOrderNumber,
                parentAccountId, codes, id, createdBy, dtCreated, lastModifiedBy, ids.
              - Query example:
                - searchQuery=name~Premium On$currency:USD.
                - This query is translated into: find accounts whose name contains the
                  phrase/prefix 'Premium On' AND the account currency is USD.

              **Note:** Using the ~ match phrase/prefix comparator. For best results, we
              recommend treating this as a "starts with" comparator for your search query.

          sort_by: Name of the parameter on which sorting is performed. Use any field available on
              the Account entity to sort by, such as `name`, `code`, and so on.

          sort_order: Sorting order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get(
            f"/organizations/{org_id}/accounts/search",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "from_document": from_document,
                        "operator": operator,
                        "page_size": page_size,
                        "search_query": search_query,
                        "sort_by": sort_by,
                        "sort_order": sort_order,
                    },
                    account_search_params.AccountSearchParams,
                ),
            ),
            cast_to=AccountSearchResponse,
        )


class AsyncAccountsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAccountsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAccountsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAccountsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncAccountsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        org_id: str | None = None,
        code: str,
        email_address: str,
        name: str,
        address: AddressParam | NotGiven = NOT_GIVEN,
        auto_generate_statement_mode: Literal["NONE", "JSON", "JSON_AND_CSV"] | NotGiven = NOT_GIVEN,
        bill_epoch: Union[str, date] | NotGiven = NOT_GIVEN,
        config_data: Dict[str, object] | NotGiven = NOT_GIVEN,
        credit_application_order: List[Literal["PREPAYMENT", "BALANCE"]] | NotGiven = NOT_GIVEN,
        currency: str | NotGiven = NOT_GIVEN,
        custom_fields: Dict[str, Union[str, float]] | NotGiven = NOT_GIVEN,
        days_before_bill_due: int | NotGiven = NOT_GIVEN,
        parent_account_id: str | NotGiven = NOT_GIVEN,
        purchase_order_number: str | NotGiven = NOT_GIVEN,
        statement_definition_id: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AccountResponse:
        """Create a new Account within the Organization.

        Args:
          code: Code of the Account.

        This is a unique short code used for the Account.

          email_address: Contact email for the Account.

          name: Name of the Account.

          address: Contact address.

          auto_generate_statement_mode: Specify whether to auto-generate statements once Bills are approved or locked.

              - **None**. Statements will not be auto-generated.
              - **JSON**. Statements are auto-generated in JSON format.
              - **JSON and CSV**. Statements are auto-generated in both JSON and CSV formats.

          bill_epoch: Optional setting to define a _billing cycle date_, which sets the date of the
              first Bill and acts as a reference for when in the applied billing frequency
              period subsequent bills are created:

              - For example, if you attach a Plan to an Account where the Plan is configured
                for monthly billing frequency and you've defined the period the Plan will
                apply to the Account to be from January 1st, 2022 until January 1st, 2023. You
                then set a `billEpoch` date of February 15th, 2022. The first Bill will be
                created for the Account on February 15th, and subsequent Bills created on the
                15th of the months following for the remainder of the billing period - March
                15th, April 15th, and so on.
              - If not defined, then the relevant Epoch date set for the billing frequency
                period at Organization level will be used instead.
              - The date is in ISO-8601 format.

          config_data:
              Configuration data for the Account Supported settings:

              - SendBillsToThirdParties ("true"/"false")

          credit_application_order: Define the order in which any Prepayment or Balance amounts on the Account are
              to be drawn-down against for billing. Four options:

              - `"PREPAYMENT","BALANCE"`. Draw-down against Prepayment credit before Balance
                credit.
              - `"BALANCE","PREPAYMENT"`. Draw-down against Balance credit before Prepayment
                credit.
              - `"PREPAYMENT"`. Only draw-down against Prepayment credit.
              - `"BALANCE"`. Only draw-down against Balance credit.

              **NOTES:**

              - Any setting you define here overrides the setting for credit application order
                at Organization level.
              - If the Account belongs to a Parent/Child Account hierarchy, then the
                `creditApplicationOrder` settings are not available, and the draw-down order
                defaults always to Prepayment then Balance order.

          currency:
              Account level billing currency, such as USD or GBP. Optional attribute:

              - If you define an Account currency, this will be used for bills.
              - If you do not define a currency, the billing currency defined at
                Organizational level will be used.

              **Note:** If you've attached a Plan to the Account that uses a different
              currency to the billing currency, then you must add the relevant currency
              conversion rate at Organization level to ensure the billing process can convert
              line items calculated using the Plan currency into the selected billing
              currency. If you don't add these conversion rates, then bills will fail for the
              Account.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          days_before_bill_due: Enter the number of days after the Bill generation date that you want to show on
              Bills as the due date.

              **Note:** If you define `daysBeforeBillDue` at individual Account level, this
              will take precedence over any `daysBeforeBillDue` setting defined at
              Organization level.

          parent_account_id: Parent Account ID, or null if this Account does not have a parent.

          purchase_order_number: Purchase Order Number of the Account.

              Optional attribute - allows you to set a purchase order number that comes
              through into invoicing. For example, your financial systems might require this
              as a reference for clearing payments.

          statement_definition_id: The UUID of the statement definition used when Bill statements are generated for
              the Account. If no statement definition is specified for the Account, the
              statement definition specified at Organizational level is used.

              Bill statements can be used as informative backing sheets to invoices. Based on
              the usage breakdown defined in the statement definition, generated statements
              give a breakdown of usage charges on Account Bills, which helps customers better
              understand usage charges incurred over the billing period.

              See
              [Working with Bill Statements](https://www.m3ter.com/docs/guides/running-viewing-and-managing-bills/working-with-bill-statements)
              in the m3ter documentation for more details.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._post(
            f"/organizations/{org_id}/accounts",
            body=await async_maybe_transform(
                {
                    "code": code,
                    "email_address": email_address,
                    "name": name,
                    "address": address,
                    "auto_generate_statement_mode": auto_generate_statement_mode,
                    "bill_epoch": bill_epoch,
                    "config_data": config_data,
                    "credit_application_order": credit_application_order,
                    "currency": currency,
                    "custom_fields": custom_fields,
                    "days_before_bill_due": days_before_bill_due,
                    "parent_account_id": parent_account_id,
                    "purchase_order_number": purchase_order_number,
                    "statement_definition_id": statement_definition_id,
                    "version": version,
                },
                account_create_params.AccountCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AccountResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AccountResponse:
        """
        Retrieve the Account with the given Account UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/accounts/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AccountResponse,
        )

    async def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        code: str,
        email_address: str,
        name: str,
        address: AddressParam | NotGiven = NOT_GIVEN,
        auto_generate_statement_mode: Literal["NONE", "JSON", "JSON_AND_CSV"] | NotGiven = NOT_GIVEN,
        bill_epoch: Union[str, date] | NotGiven = NOT_GIVEN,
        config_data: Dict[str, object] | NotGiven = NOT_GIVEN,
        credit_application_order: List[Literal["PREPAYMENT", "BALANCE"]] | NotGiven = NOT_GIVEN,
        currency: str | NotGiven = NOT_GIVEN,
        custom_fields: Dict[str, Union[str, float]] | NotGiven = NOT_GIVEN,
        days_before_bill_due: int | NotGiven = NOT_GIVEN,
        parent_account_id: str | NotGiven = NOT_GIVEN,
        purchase_order_number: str | NotGiven = NOT_GIVEN,
        statement_definition_id: str | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AccountResponse:
        """
        Update the Account with the given Account UUID.

        **Note:** If you have created Custom Fields for an Account, when you use this
        endpoint to update the Account, use the `customFields` parameter to preserve
        those Custom Fields. If you omit them from the update request, they will be
        lost.

        Args:
          code: Code of the Account. This is a unique short code used for the Account.

          email_address: Contact email for the Account.

          name: Name of the Account.

          address: Contact address.

          auto_generate_statement_mode: Specify whether to auto-generate statements once Bills are approved or locked.

              - **None**. Statements will not be auto-generated.
              - **JSON**. Statements are auto-generated in JSON format.
              - **JSON and CSV**. Statements are auto-generated in both JSON and CSV formats.

          bill_epoch: Optional setting to define a _billing cycle date_, which sets the date of the
              first Bill and acts as a reference for when in the applied billing frequency
              period subsequent bills are created:

              - For example, if you attach a Plan to an Account where the Plan is configured
                for monthly billing frequency and you've defined the period the Plan will
                apply to the Account to be from January 1st, 2022 until January 1st, 2023. You
                then set a `billEpoch` date of February 15th, 2022. The first Bill will be
                created for the Account on February 15th, and subsequent Bills created on the
                15th of the months following for the remainder of the billing period - March
                15th, April 15th, and so on.
              - If not defined, then the relevant Epoch date set for the billing frequency
                period at Organization level will be used instead.
              - The date is in ISO-8601 format.

          config_data:
              Configuration data for the Account Supported settings:

              - SendBillsToThirdParties ("true"/"false")

          credit_application_order: Define the order in which any Prepayment or Balance amounts on the Account are
              to be drawn-down against for billing. Four options:

              - `"PREPAYMENT","BALANCE"`. Draw-down against Prepayment credit before Balance
                credit.
              - `"BALANCE","PREPAYMENT"`. Draw-down against Balance credit before Prepayment
                credit.
              - `"PREPAYMENT"`. Only draw-down against Prepayment credit.
              - `"BALANCE"`. Only draw-down against Balance credit.

              **NOTES:**

              - Any setting you define here overrides the setting for credit application order
                at Organization level.
              - If the Account belongs to a Parent/Child Account hierarchy, then the
                `creditApplicationOrder` settings are not available, and the draw-down order
                defaults always to Prepayment then Balance order.

          currency:
              Account level billing currency, such as USD or GBP. Optional attribute:

              - If you define an Account currency, this will be used for bills.
              - If you do not define a currency, the billing currency defined at
                Organizational level will be used.

              **Note:** If you've attached a Plan to the Account that uses a different
              currency to the billing currency, then you must add the relevant currency
              conversion rate at Organization level to ensure the billing process can convert
              line items calculated using the Plan currency into the selected billing
              currency. If you don't add these conversion rates, then bills will fail for the
              Account.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          days_before_bill_due: Enter the number of days after the Bill generation date that you want to show on
              Bills as the due date.

              **Note:** If you define `daysBeforeBillDue` at individual Account level, this
              will take precedence over any `daysBeforeBillDue` setting defined at
              Organization level.

          parent_account_id: Parent Account ID, or null if this Account does not have a parent.

          purchase_order_number: Purchase Order Number of the Account.

              Optional attribute - allows you to set a purchase order number that comes
              through into invoicing. For example, your financial systems might require this
              as a reference for clearing payments.

          statement_definition_id: The UUID of the statement definition used when Bill statements are generated for
              the Account. If no statement definition is specified for the Account, the
              statement definition specified at Organizational level is used.

              Bill statements can be used as informative backing sheets to invoices. Based on
              the usage breakdown defined in the statement definition, generated statements
              give a breakdown of usage charges on Account Bills, which helps customers better
              understand usage charges incurred over the billing period.

              See
              [Working with Bill Statements](https://www.m3ter.com/docs/guides/running-viewing-and-managing-bills/working-with-bill-statements)
              in the m3ter documentation for more details.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/organizations/{org_id}/accounts/{id}",
            body=await async_maybe_transform(
                {
                    "code": code,
                    "email_address": email_address,
                    "name": name,
                    "address": address,
                    "auto_generate_statement_mode": auto_generate_statement_mode,
                    "bill_epoch": bill_epoch,
                    "config_data": config_data,
                    "credit_application_order": credit_application_order,
                    "currency": currency,
                    "custom_fields": custom_fields,
                    "days_before_bill_due": days_before_bill_due,
                    "parent_account_id": parent_account_id,
                    "purchase_order_number": purchase_order_number,
                    "statement_definition_id": statement_definition_id,
                    "version": version,
                },
                account_update_params.AccountUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AccountResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        codes: List[str] | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[AccountResponse, AsyncCursor[AccountResponse]]:
        """
        Retrieve a list of Accounts that can be filtered by Account ID or Account Code.

        Args:
          codes: List of Account Codes to retrieve. These are unique short codes for each
              Account.

          ids: List of Account IDs to retrieve.

          next_token: `nextToken` for multi-page retrievals.

          page_size: Number of accounts to retrieve per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/accounts",
            page=AsyncCursor[AccountResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "codes": codes,
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    account_list_params.AccountListParams,
                ),
            ),
            model=AccountResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AccountResponse:
        """Delete the Account with the given UUID.

        This may fail if there are any
        AccountPlans that reference the Account being deleted.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/organizations/{org_id}/accounts/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AccountResponse,
        )

    async def end_date_billing_entities(
        self,
        id: str,
        *,
        org_id: str | None = None,
        billing_entities: List[Literal["CONTRACT", "ACCOUNTPLAN", "PREPAYMENT", "PRICINGS", "COUNTER_PRICINGS"]],
        end_date: Union[str, datetime],
        apply_to_children: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AccountEndDateBillingEntitiesResponse:
        """
        Apply the specified end-date to billing entities associated with an Account.

        **NOTE:**

        - When you successfully end-date billing entities, the version number of each
          entity is incremented.

        Args:
          billing_entities: Defines which billing entities associated with the Account will have the
              specified end-date applied. For example, if you want the specified end-date to
              be applied to all Prepayments/Commitments created for the Account use
              `"PREPAYMENT"`.

          end_date: The end date and time applied to the specified billing entities _(in ISO 8601
              format)_.

          apply_to_children: A Boolean TRUE/FALSE flag. For Parent Accounts, set to TRUE if you want the
              specified end-date to be applied to any billing entities associated with Child
              Accounts. _(Optional)_

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/organizations/{org_id}/accounts/{id}/enddatebillingentities",
            body=await async_maybe_transform(
                {
                    "billing_entities": billing_entities,
                    "end_date": end_date,
                    "apply_to_children": apply_to_children,
                },
                account_end_date_billing_entities_params.AccountEndDateBillingEntitiesParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AccountEndDateBillingEntitiesResponse,
        )

    async def get_children(
        self,
        id: str,
        *,
        org_id: str | None = None,
        next_token: Optional[str] | NotGiven = NOT_GIVEN,
        page_size: Optional[int] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AccountResponse:
        """
        Retrieve a list of Accounts that are children of the specified Account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/accounts/{id}/children",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    account_get_children_params.AccountGetChildrenParams,
                ),
            ),
            cast_to=AccountResponse,
        )

    async def search(
        self,
        *,
        org_id: str | None = None,
        from_document: int | NotGiven = NOT_GIVEN,
        operator: Literal["AND", "OR"] | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        search_query: str | NotGiven = NOT_GIVEN,
        sort_by: str | NotGiven = NOT_GIVEN,
        sort_order: Literal["ASC", "DESC"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AccountSearchResponse:
        """
        Search for Account entities.

        This endpoint executes a search query for Accounts based on the user specified
        search criteria. The search query is customizable, allowing for complex nested
        conditions and sorting. The returned list of Accounts can be paginated for
        easier management.

        Args:
          from_document: `fromDocument` for multi page retrievals.

          operator: Search Operator to be used while querying search.

          page_size: Number of Accounts to retrieve per page.

              **NOTE:** If not defined, default is 10.

          search_query:
              Query for data using special syntax:

              - Query parameters should be delimited using the $ (dollar sign).
              - Allowed comparators are:
                - (greater than) >
                - (greater than or equal to) >=
                - (equal to) :
                - (less than) <
                - (less than or equal to) <=
                - (match phrase/prefix) ~
              - Allowed parameters are: name, code, currency, purchaseOrderNumber,
                parentAccountId, codes, id, createdBy, dtCreated, lastModifiedBy, ids.
              - Query example:
                - searchQuery=name~Premium On$currency:USD.
                - This query is translated into: find accounts whose name contains the
                  phrase/prefix 'Premium On' AND the account currency is USD.

              **Note:** Using the ~ match phrase/prefix comparator. For best results, we
              recommend treating this as a "starts with" comparator for your search query.

          sort_by: Name of the parameter on which sorting is performed. Use any field available on
              the Account entity to sort by, such as `name`, `code`, and so on.

          sort_order: Sorting order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._get(
            f"/organizations/{org_id}/accounts/search",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "from_document": from_document,
                        "operator": operator,
                        "page_size": page_size,
                        "search_query": search_query,
                        "sort_by": sort_by,
                        "sort_order": sort_order,
                    },
                    account_search_params.AccountSearchParams,
                ),
            ),
            cast_to=AccountSearchResponse,
        )


class AccountsResourceWithRawResponse:
    def __init__(self, accounts: AccountsResource) -> None:
        self._accounts = accounts

        self.create = to_raw_response_wrapper(
            accounts.create,
        )
        self.retrieve = to_raw_response_wrapper(
            accounts.retrieve,
        )
        self.update = to_raw_response_wrapper(
            accounts.update,
        )
        self.list = to_raw_response_wrapper(
            accounts.list,
        )
        self.delete = to_raw_response_wrapper(
            accounts.delete,
        )
        self.end_date_billing_entities = to_raw_response_wrapper(
            accounts.end_date_billing_entities,
        )
        self.get_children = to_raw_response_wrapper(
            accounts.get_children,
        )
        self.search = to_raw_response_wrapper(
            accounts.search,
        )


class AsyncAccountsResourceWithRawResponse:
    def __init__(self, accounts: AsyncAccountsResource) -> None:
        self._accounts = accounts

        self.create = async_to_raw_response_wrapper(
            accounts.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            accounts.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            accounts.update,
        )
        self.list = async_to_raw_response_wrapper(
            accounts.list,
        )
        self.delete = async_to_raw_response_wrapper(
            accounts.delete,
        )
        self.end_date_billing_entities = async_to_raw_response_wrapper(
            accounts.end_date_billing_entities,
        )
        self.get_children = async_to_raw_response_wrapper(
            accounts.get_children,
        )
        self.search = async_to_raw_response_wrapper(
            accounts.search,
        )


class AccountsResourceWithStreamingResponse:
    def __init__(self, accounts: AccountsResource) -> None:
        self._accounts = accounts

        self.create = to_streamed_response_wrapper(
            accounts.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            accounts.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            accounts.update,
        )
        self.list = to_streamed_response_wrapper(
            accounts.list,
        )
        self.delete = to_streamed_response_wrapper(
            accounts.delete,
        )
        self.end_date_billing_entities = to_streamed_response_wrapper(
            accounts.end_date_billing_entities,
        )
        self.get_children = to_streamed_response_wrapper(
            accounts.get_children,
        )
        self.search = to_streamed_response_wrapper(
            accounts.search,
        )


class AsyncAccountsResourceWithStreamingResponse:
    def __init__(self, accounts: AsyncAccountsResource) -> None:
        self._accounts = accounts

        self.create = async_to_streamed_response_wrapper(
            accounts.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            accounts.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            accounts.update,
        )
        self.list = async_to_streamed_response_wrapper(
            accounts.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            accounts.delete,
        )
        self.end_date_billing_entities = async_to_streamed_response_wrapper(
            accounts.end_date_billing_entities,
        )
        self.get_children = async_to_streamed_response_wrapper(
            accounts.get_children,
        )
        self.search = async_to_streamed_response_wrapper(
            accounts.search,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["CounterListParams"]


class CounterListParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    codes: List[str]
    """List of Counter codes to retrieve.

    These are unique short codes to identify each Counter.
    """

    ids: List[str]
    """List of Counter IDs to retrieve."""

    next_token: Annotated[str, PropertyInfo(alias="nextToken")]
    """NextToken for multi page retrievals."""

    page_size: Annotated[int, PropertyInfo(alias="pageSize")]
    """Number of Counters to retrieve per page"""

    product_id: Annotated[List[str], PropertyInfo(alias="productId")]
    """List of Products UUIDs to retrieve Counters for."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["EventGetTypesResponse"]


class EventGetTypesResponse(BaseModel):
    events: Optional[List[str]] = None
    """
    An array containing a list of all Event Types for which Notification rules can
    be configured. Each Event Type is represented by a string.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import date
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["BillConfigUpdateParams"]


class BillConfigUpdateParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    bill_lock_date: Annotated[Union[str, date], PropertyInfo(alias="billLockDate", format="iso8601")]
    """The global lock date when all Bills will be locked _(in ISO 8601 format)_.

    For example: `"2024-03-01"`.
    """

    version: int
    """The version number:

    - Default value when newly created is one.
    - On Update, version is required and must match the existing version because a
      check is performed to ensure sequential versioning is preserved. Version is
      incremented by 1 and listed in the response
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Iterable
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["AggregationUpdateParams"]


class AggregationUpdateParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    aggregation: Required[Literal["SUM", "MIN", "MAX", "COUNT", "LATEST", "MEAN", "UNIQUE", "CUSTOM_SQL"]]
    """
    Specifies the computation method applied to usage data collected in
    `targetField`. Aggregation unit value depends on the **Category** configured for
    the selected targetField.

    Enum:

    - **SUM**. Adds the values. Can be applied to a **Measure**, **Income**, or
      **Cost** `targetField`.

    - **MIN**. Uses the minimum value. Can be applied to a **Measure**, **Income**,
      or **Cost** `targetField`.

    - **MAX**. Uses the maximum value. Can be applied to a **Measure**, **Income**,
      or **Cost** `targetField`.

    - **COUNT**. Counts the number of values. Can be applied to a **Measure**,
      **Income**, or **Cost** `targetField`.

    - **LATEST**. Uses the most recent value. Can be applied to a **Measure**,
      **Income**, or **Cost** `targetField`. Note: Based on the timestamp (`ts`)
      value of usage data measurement submissions. If using this method, please
      ensure _distinct_ `ts` values are used for usage data measurment submissions.

    - **MEAN**. Uses the arithmetic mean of the values. Can be applied to a
      **Measure**, **Income**, or **Cost** `targetField`.

    - **UNIQUE**. Uses unique values and returns a count of the number of unique
      values. Can be applied to a **Metadata** `targetField`.
    """

    meter_id: Required[Annotated[str, PropertyInfo(alias="meterId")]]
    """The UUID of the Meter used as the source of usage data for the Aggregation.

    Each Aggregation is a child of a Meter, so the Meter must be selected.
    """

    name: Required[str]
    """Descriptive name for the Aggregation."""

    quantity_per_unit: Required[Annotated[float, PropertyInfo(alias="quantityPerUnit")]]
    """Defines how much of a quantity equates to 1 unit.

    Used when setting the price per unit for billing purposes - if charging for
    kilobytes per second (KiBy/s) at rate of $0.25 per 500 KiBy/s, then set
    quantityPerUnit to 500 and price Plan at $0.25 per unit.

    **Note:** If `quantityPerUnit` is set to a value other than one, `rounding` is
    typically set to `"UP"`.
    """

    rounding: Required[Literal["UP", "DOWN", "NEAREST", "NONE"]]
    """
    Specifies how you want to deal with non-integer, fractional number Aggregation
    values.

    **NOTES:**

    - **NEAREST** rounds to the nearest half: 5.1 is rounded to 5, and 3.5 is
      rounded to 4.
    - Also used in combination with `quantityPerUnit`. Rounds the number of units
      after `quantityPerUnit` is applied. If you set `quantityPerUnit` to a value
      other than one, you would typically set Rounding to **UP**. For example,
      suppose you charge by kilobytes per second (KiBy/s), set `quantityPerUnit` =
      500, and set charge rate at $0.25 per unit used. If your customer used 48,900
      KiBy/s in a billing period, the charge would be 48,900 / 500 = 97.8 rounded up
      to 98 \\** 0.25 = $2.45.

    Enum: ???UP??? ???DOWN??? ???NEAREST??? ???NONE???
    """

    target_field: Required[Annotated[str, PropertyInfo(alias="targetField")]]
    """
    `Code` of the target `dataField` or `derivedField` on the Meter used as the
    basis for the Aggregation.
    """

    unit: Required[str]
    """
    User defined label for units shown for Bill line items, indicating to your
    customers what they are being charged for.
    """

    accounting_product_id: Annotated[str, PropertyInfo(alias="accountingProductId")]
    """
    Optional Product ID this Aggregation should be attributed to for accounting
    purposes
    """

    code: str
    """Code of the new Aggregation. A unique short code to identify the Aggregation."""

    custom_fields: Annotated[Dict[str, Union[str, float]], PropertyInfo(alias="customFields")]

    custom_sql: Annotated[str, PropertyInfo(alias="customSql")]

    default_value: Annotated[float, PropertyInfo(alias="defaultValue")]
    """Aggregation value used when no usage data is available to be aggregated.

    _(Optional)_.

    **Note:** Set to 0, if you expect to reference the Aggregation in a Compound
    Aggregation. This ensures that any null values are passed in correctly to the
    Compound Aggregation calculation with a value = 0.
    """

    segmented_fields: Annotated[List[str], PropertyInfo(alias="segmentedFields")]
    """_(Optional)_.

    Used when creating a segmented Aggregation, which segments the usage data
    collected by a single Meter. Works together with `segments`.

    Enter the `Codes` of the fields in the target Meter to use for segmentation
    purposes.

    String `dataFields` on the target Meter can be segmented. Any string
    `derivedFields` on the target Meter, such as one that concatenates two string
    `dataFields`, can also be segmented.
    """

    segments: Iterable[Dict[str, str]]
    """_(Optional)_.

    Used when creating a segmented Aggregation, which segments the usage data
    collected by a single Meter. Works together with `segmentedFields`.

    Enter the values that are to be used as the segments, read from the fields in
    the meter pointed at by `segmentedFields`.

    Note that you can use _wildcards_ or _defaults_ when setting up segment values.
    For more details on how to do this with an example, see
    [Using Wildcards - API Calls](https://www.m3ter.com/docs/guides/setting-up-usage-data-meters-and-aggregations/segmented-aggregations#using-wildcards---api-calls)
    in our main User Docs.
    """

    version: int
    """The version number of the entity:

    - **Create entity:** Not valid for initial insertion of new entity - _do not use
      for Create_. On initial Create, version is set at 1 and listed in the
      response.
    - **Update Entity:** On Update, version is required and must match the existing
      version because a check is performed to ensure sequential versioning is
      preserved. Version is incremented by 1 and listed in the response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import date, datetime

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["BillConfigResponse"]


class BillConfigResponse(BaseModel):
    id: Optional[str] = None
    """The Organization UUID.

    The Organization represents your company as a direct customer of the m3ter
    service.
    """

    bill_lock_date: Optional[date] = FieldInfo(alias="billLockDate", default=None)
    """The global lock date _(in ISO 8601 format)_ when all Bills will be locked.

    For example: `"2024-03-01"`.
    """

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """The id of the user who created this bill config."""

    dt_created: Optional[datetime] = FieldInfo(alias="dtCreated", default=None)
    """The DateTime _(in ISO-8601 format)_ when the bill config was first created."""

    dt_last_modified: Optional[datetime] = FieldInfo(alias="dtLastModified", default=None)
    """The DateTime _(in ISO-8601 format)_ when the bill config was last modified."""

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """The id of the user who last modified this bill config."""

    version: Optional[int] = None
    """The version number:

    - Default value when newly created is one.
    - Incremented by 1 each time it is updated.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Optional
from typing_extensions import Literal

import httpx

from ...types import bill_list_params, bill_search_params, bill_approve_params, bill_update_status_params
from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from .line_items import (
    LineItemsResource,
    AsyncLineItemsResource,
    LineItemsResourceWithRawResponse,
    AsyncLineItemsResourceWithRawResponse,
    LineItemsResourceWithStreamingResponse,
    AsyncLineItemsResourceWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncCursor, AsyncCursor
from ..._base_client import AsyncPaginator, make_request_options
from .debit_line_items import (
    DebitLineItemsResource,
    AsyncDebitLineItemsResource,
    DebitLineItemsResourceWithRawResponse,
    AsyncDebitLineItemsResourceWithRawResponse,
    DebitLineItemsResourceWithStreamingResponse,
    AsyncDebitLineItemsResourceWithStreamingResponse,
)
from .credit_line_items import (
    CreditLineItemsResource,
    AsyncCreditLineItemsResource,
    CreditLineItemsResourceWithRawResponse,
    AsyncCreditLineItemsResourceWithRawResponse,
    CreditLineItemsResourceWithStreamingResponse,
    AsyncCreditLineItemsResourceWithStreamingResponse,
)
from ...types.bill_response import BillResponse
from ...types.bill_search_response import BillSearchResponse
from ...types.bill_approve_response import BillApproveResponse

__all__ = ["BillsResource", "AsyncBillsResource"]


class BillsResource(SyncAPIResource):
    @cached_property
    def credit_line_items(self) -> CreditLineItemsResource:
        return CreditLineItemsResource(self._client)

    @cached_property
    def debit_line_items(self) -> DebitLineItemsResource:
        return DebitLineItemsResource(self._client)

    @cached_property
    def line_items(self) -> LineItemsResource:
        return LineItemsResource(self._client)

    @cached_property
    def with_raw_response(self) -> BillsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return BillsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BillsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return BillsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BillResponse:
        """
        Retrieve the Bill with the given UUID.

        This endpoint retrieves the Bill with the given unique identifier (UUID) and
        specific Organization.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/bills/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BillResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        account_id: str | NotGiven = NOT_GIVEN,
        bill_date: str | NotGiven = NOT_GIVEN,
        bill_date_end: str | NotGiven = NOT_GIVEN,
        bill_date_start: str | NotGiven = NOT_GIVEN,
        billing_frequency: Optional[str] | NotGiven = NOT_GIVEN,
        exclude_line_items: bool | NotGiven = NOT_GIVEN,
        external_invoice_date_end: str | NotGiven = NOT_GIVEN,
        external_invoice_date_start: str | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        include_bill_total: bool | NotGiven = NOT_GIVEN,
        locked: bool | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        status: Literal["PENDING", "APPROVED"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursor[BillResponse]:
        """
        Retrieve a list of Bills.

        This endpoint retrieves a list of all Bills for the given Account within the
        specified Organization. Optional filters can be applied such as by date range,
        lock status, or other attributes. The list can also be paginated for easier
        management.

        Args:
          account_id: Optional filter. An Account ID - returns the Bills for the single specified
              Account.

          bill_date: The specific date in ISO 8601 format for which you want to retrieve Bills.

          bill_date_end: Only include Bills with bill dates earlier than this date.

          bill_date_start: Only include Bills with bill dates equal to or later than this date.

          exclude_line_items: Exclude Line Items

          external_invoice_date_end: Only include Bills with external invoice dates earlier than this date.

          external_invoice_date_start: Only include Bills with external invoice dates equal to or later than this date.

          ids: Optional filter. The list of Bill IDs to retrieve.

          include_bill_total: Include Bill Total

          locked: Boolean flag specifying whether to include Bills with "locked" status.

              - **TRUE** - the list inlcudes "locked" Bills.
              - **FALSE** - excludes "locked" Bills from the list.

          next_token: The `nextToken` for multi-page retrievals. It is used to fetch the next page of
              Bills in a paginated list.

          page_size: Specifies the maximum number of Bills to retrieve per page.

          status: Only include Bills having the given status

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/bills",
            page=SyncCursor[BillResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "bill_date": bill_date,
                        "bill_date_end": bill_date_end,
                        "bill_date_start": bill_date_start,
                        "billing_frequency": billing_frequency,
                        "exclude_line_items": exclude_line_items,
                        "external_invoice_date_end": external_invoice_date_end,
                        "external_invoice_date_start": external_invoice_date_start,
                        "ids": ids,
                        "include_bill_total": include_bill_total,
                        "locked": locked,
                        "next_token": next_token,
                        "page_size": page_size,
                        "status": status,
                    },
                    bill_list_params.BillListParams,
                ),
            ),
            model=BillResponse,
        )

    def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BillResponse:
        """
        Delete the Bill with the given UUID.

        This endpoint deletes the specified Bill with the given unique identifier. Use
        with caution since deleted Bills cannot be recovered. Suitable for removing
        incorrect or obsolete Bills, and for Bills that have not been sent to customers.
        Where end-customer invoices for Bills have been sent to customers, Bills should
        not be deleted to ensure you have an audit trail of how the invoice was created.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/organizations/{org_id}/bills/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BillResponse,
        )

    def approve(
        self,
        *,
        org_id: str | None = None,
        bill_ids: List[str],
        account_ids: str | NotGiven = NOT_GIVEN,
        external_invoice_date_end: str | NotGiven = NOT_GIVEN,
        external_invoice_date_start: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BillApproveResponse:
        """
        Approve multiple Bills for the specified Organization based on the given
        criteria.

        This endpoint allows you to change currently _Pending_ Bills to _Approved_
        status for further processing.

        Query Parameters:

        - Use `accountIds` to approve Bills for specifed Accounts.

        Request Body Schema Parameter:

        - Use `billIds` to specify a collection of Bills for batch approval.

        **Important!** If you use the `billIds` Request Body Schema parameter, any Query
        parameters you might have also used are ignored when the call is processed.

        Args:
          bill_ids: Use to specify a collection of Bills by their IDs for batch approval

          account_ids: List of Account IDs to filter Bills. This allows you to approve Bills for
              specific Accounts within the Organization.

          external_invoice_date_end: End date for filtering Bills by external invoice date. Includes Bills with dates
              earlier than this date.

          external_invoice_date_start: Start date for filtering Bills by external invoice date. Includes Bills with
              dates equal to or later than this date.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._post(
            f"/organizations/{org_id}/bills/approve",
            body=maybe_transform({"bill_ids": bill_ids}, bill_approve_params.BillApproveParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_ids": account_ids,
                        "external_invoice_date_end": external_invoice_date_end,
                        "external_invoice_date_start": external_invoice_date_start,
                    },
                    bill_approve_params.BillApproveParams,
                ),
            ),
            cast_to=BillApproveResponse,
        )

    def latest_by_account(
        self,
        account_id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BillResponse:
        """
        Retrieve the latest Bill for the given Account.

        This endpoint retrieves the latest Bill for the given Account in the specified
        Organization. It facilitates tracking of the most recent charges and consumption
        details.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            f"/organizations/{org_id}/bills/latest/{account_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BillResponse,
        )

    def lock(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BillResponse:
        """Lock the specific Bill identified by the given UUID.

        Once a Bill is locked, no
        further changes can be made to it.

        **NOTE:** You cannot lock a Bill whose current status is `PENDING`. You will
        receive an error message if you try to do this. You must first use the
        [Approve Bills](https://www.m3ter.com/docs/api#tag/Bill/operation/ApproveBills)
        call to approve a Bill before you can lock it.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/organizations/{org_id}/bills/{id}/lock",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BillResponse,
        )

    def search(
        self,
        *,
        org_id: str | None = None,
        from_document: int | NotGiven = NOT_GIVEN,
        operator: Literal["AND", "OR"] | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        search_query: str | NotGiven = NOT_GIVEN,
        sort_by: str | NotGiven = NOT_GIVEN,
        sort_order: Literal["ASC", "DESC"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BillSearchResponse:
        """
        Search for Bill entities.

        This endpoint executes a search query for Bills based on the user specified
        search criteria. The search query is customizable, allowing for complex nested
        conditions and sorting. The returned list of Bills can be paginated for easier
        management.

        Args:
          from_document: `fromDocument` for multi page retrievals.

          operator: Search Operator to be used while querying search.

          page_size: Number of Bills to retrieve per page.

              **NOTE:** If not defined, default is 10.

          search_query:
              Query for data using special syntax:

              - Query parameters should be delimited using $ (dollar sign).
              - Allowed comparators are:
                - (greater than) >
                - (greater than or equal to) >=
                - (equal to) :
                - (less than) <
                - (less than or equal to) <=
                - (match phrase/prefix) ~
              - Allowed parameters: accountId, locked, billDate, startDate, endDate, dueDate,
                billingFrequency, id, createdBy, dtCreated, lastModifiedBy, ids.
              - Query example:
                - searchQuery=startDate>2023-01-01$accountId:62eaad67-5790-407e-b853-881564f0e543.
                - This query is translated into: find Bills that startDate is older than
                  2023-01-01 AND accountId is equal to 62eaad67-5790-407e-b853-881564f0e543.

              **Note:** Using the ~ match phrase/prefix comparator. For best results, we
              recommend treating this as a "starts with" comparator for your search query.

          sort_by: Name of the parameter on which sorting is performed. Use any field available on
              the Bill entity to sort by, such as `accountId`, `endDate`, and so on.

          sort_order: Sorting order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get(
            f"/organizations/{org_id}/bills/search",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "from_document": from_document,
                        "operator": operator,
                        "page_size": page_size,
                        "search_query": search_query,
                        "sort_by": sort_by,
                        "sort_order": sort_order,
                    },
                    bill_search_params.BillSearchParams,
                ),
            ),
            cast_to=BillSearchResponse,
        )

    def update_status(
        self,
        id: str,
        *,
        org_id: str | None = None,
        status: Literal["PENDING", "APPROVED"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BillResponse:
        """
        Updates the status of a specified Bill with the given Bill ID.

        This endpoint allows you to transition a Bill's status through various stages,
        such as from "Pending" to "Approved".

        Args:
          status: The new status you want to assign to the Bill. Must be one "Pending" or
              "Approved".

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/organizations/{org_id}/bills/{id}/status",
            body=maybe_transform({"status": status}, bill_update_status_params.BillUpdateStatusParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BillResponse,
        )


class AsyncBillsResource(AsyncAPIResource):
    @cached_property
    def credit_line_items(self) -> AsyncCreditLineItemsResource:
        return AsyncCreditLineItemsResource(self._client)

    @cached_property
    def debit_line_items(self) -> AsyncDebitLineItemsResource:
        return AsyncDebitLineItemsResource(self._client)

    @cached_property
    def line_items(self) -> AsyncLineItemsResource:
        return AsyncLineItemsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncBillsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncBillsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBillsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncBillsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BillResponse:
        """
        Retrieve the Bill with the given UUID.

        This endpoint retrieves the Bill with the given unique identifier (UUID) and
        specific Organization.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/bills/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BillResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        account_id: str | NotGiven = NOT_GIVEN,
        bill_date: str | NotGiven = NOT_GIVEN,
        bill_date_end: str | NotGiven = NOT_GIVEN,
        bill_date_start: str | NotGiven = NOT_GIVEN,
        billing_frequency: Optional[str] | NotGiven = NOT_GIVEN,
        exclude_line_items: bool | NotGiven = NOT_GIVEN,
        external_invoice_date_end: str | NotGiven = NOT_GIVEN,
        external_invoice_date_start: str | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        include_bill_total: bool | NotGiven = NOT_GIVEN,
        locked: bool | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        status: Literal["PENDING", "APPROVED"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[BillResponse, AsyncCursor[BillResponse]]:
        """
        Retrieve a list of Bills.

        This endpoint retrieves a list of all Bills for the given Account within the
        specified Organization. Optional filters can be applied such as by date range,
        lock status, or other attributes. The list can also be paginated for easier
        management.

        Args:
          account_id: Optional filter. An Account ID - returns the Bills for the single specified
              Account.

          bill_date: The specific date in ISO 8601 format for which you want to retrieve Bills.

          bill_date_end: Only include Bills with bill dates earlier than this date.

          bill_date_start: Only include Bills with bill dates equal to or later than this date.

          exclude_line_items: Exclude Line Items

          external_invoice_date_end: Only include Bills with external invoice dates earlier than this date.

          external_invoice_date_start: Only include Bills with external invoice dates equal to or later than this date.

          ids: Optional filter. The list of Bill IDs to retrieve.

          include_bill_total: Include Bill Total

          locked: Boolean flag specifying whether to include Bills with "locked" status.

              - **TRUE** - the list inlcudes "locked" Bills.
              - **FALSE** - excludes "locked" Bills from the list.

          next_token: The `nextToken` for multi-page retrievals. It is used to fetch the next page of
              Bills in a paginated list.

          page_size: Specifies the maximum number of Bills to retrieve per page.

          status: Only include Bills having the given status

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/bills",
            page=AsyncCursor[BillResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "bill_date": bill_date,
                        "bill_date_end": bill_date_end,
                        "bill_date_start": bill_date_start,
                        "billing_frequency": billing_frequency,
                        "exclude_line_items": exclude_line_items,
                        "external_invoice_date_end": external_invoice_date_end,
                        "external_invoice_date_start": external_invoice_date_start,
                        "ids": ids,
                        "include_bill_total": include_bill_total,
                        "locked": locked,
                        "next_token": next_token,
                        "page_size": page_size,
                        "status": status,
                    },
                    bill_list_params.BillListParams,
                ),
            ),
            model=BillResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BillResponse:
        """
        Delete the Bill with the given UUID.

        This endpoint deletes the specified Bill with the given unique identifier. Use
        with caution since deleted Bills cannot be recovered. Suitable for removing
        incorrect or obsolete Bills, and for Bills that have not been sent to customers.
        Where end-customer invoices for Bills have been sent to customers, Bills should
        not be deleted to ensure you have an audit trail of how the invoice was created.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/organizations/{org_id}/bills/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BillResponse,
        )

    async def approve(
        self,
        *,
        org_id: str | None = None,
        bill_ids: List[str],
        account_ids: str | NotGiven = NOT_GIVEN,
        external_invoice_date_end: str | NotGiven = NOT_GIVEN,
        external_invoice_date_start: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BillApproveResponse:
        """
        Approve multiple Bills for the specified Organization based on the given
        criteria.

        This endpoint allows you to change currently _Pending_ Bills to _Approved_
        status for further processing.

        Query Parameters:

        - Use `accountIds` to approve Bills for specifed Accounts.

        Request Body Schema Parameter:

        - Use `billIds` to specify a collection of Bills for batch approval.

        **Important!** If you use the `billIds` Request Body Schema parameter, any Query
        parameters you might have also used are ignored when the call is processed.

        Args:
          bill_ids: Use to specify a collection of Bills by their IDs for batch approval

          account_ids: List of Account IDs to filter Bills. This allows you to approve Bills for
              specific Accounts within the Organization.

          external_invoice_date_end: End date for filtering Bills by external invoice date. Includes Bills with dates
              earlier than this date.

          external_invoice_date_start: Start date for filtering Bills by external invoice date. Includes Bills with
              dates equal to or later than this date.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._post(
            f"/organizations/{org_id}/bills/approve",
            body=await async_maybe_transform({"bill_ids": bill_ids}, bill_approve_params.BillApproveParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "account_ids": account_ids,
                        "external_invoice_date_end": external_invoice_date_end,
                        "external_invoice_date_start": external_invoice_date_start,
                    },
                    bill_approve_params.BillApproveParams,
                ),
            ),
            cast_to=BillApproveResponse,
        )

    async def latest_by_account(
        self,
        account_id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BillResponse:
        """
        Retrieve the latest Bill for the given Account.

        This endpoint retrieves the latest Bill for the given Account in the specified
        Organization. It facilitates tracking of the most recent charges and consumption
        details.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            f"/organizations/{org_id}/bills/latest/{account_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BillResponse,
        )

    async def lock(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BillResponse:
        """Lock the specific Bill identified by the given UUID.

        Once a Bill is locked, no
        further changes can be made to it.

        **NOTE:** You cannot lock a Bill whose current status is `PENDING`. You will
        receive an error message if you try to do this. You must first use the
        [Approve Bills](https://www.m3ter.com/docs/api#tag/Bill/operation/ApproveBills)
        call to approve a Bill before you can lock it.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/organizations/{org_id}/bills/{id}/lock",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BillResponse,
        )

    async def search(
        self,
        *,
        org_id: str | None = None,
        from_document: int | NotGiven = NOT_GIVEN,
        operator: Literal["AND", "OR"] | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        search_query: str | NotGiven = NOT_GIVEN,
        sort_by: str | NotGiven = NOT_GIVEN,
        sort_order: Literal["ASC", "DESC"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BillSearchResponse:
        """
        Search for Bill entities.

        This endpoint executes a search query for Bills based on the user specified
        search criteria. The search query is customizable, allowing for complex nested
        conditions and sorting. The returned list of Bills can be paginated for easier
        management.

        Args:
          from_document: `fromDocument` for multi page retrievals.

          operator: Search Operator to be used while querying search.

          page_size: Number of Bills to retrieve per page.

              **NOTE:** If not defined, default is 10.

          search_query:
              Query for data using special syntax:

              - Query parameters should be delimited using $ (dollar sign).
              - Allowed comparators are:
                - (greater than) >
                - (greater than or equal to) >=
                - (equal to) :
                - (less than) <
                - (less than or equal to) <=
                - (match phrase/prefix) ~
              - Allowed parameters: accountId, locked, billDate, startDate, endDate, dueDate,
                billingFrequency, id, createdBy, dtCreated, lastModifiedBy, ids.
              - Query example:
                - searchQuery=startDate>2023-01-01$accountId:62eaad67-5790-407e-b853-881564f0e543.
                - This query is translated into: find Bills that startDate is older than
                  2023-01-01 AND accountId is equal to 62eaad67-5790-407e-b853-881564f0e543.

              **Note:** Using the ~ match phrase/prefix comparator. For best results, we
              recommend treating this as a "starts with" comparator for your search query.

          sort_by: Name of the parameter on which sorting is performed. Use any field available on
              the Bill entity to sort by, such as `accountId`, `endDate`, and so on.

          sort_order: Sorting order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._get(
            f"/organizations/{org_id}/bills/search",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "from_document": from_document,
                        "operator": operator,
                        "page_size": page_size,
                        "search_query": search_query,
                        "sort_by": sort_by,
                        "sort_order": sort_order,
                    },
                    bill_search_params.BillSearchParams,
                ),
            ),
            cast_to=BillSearchResponse,
        )

    async def update_status(
        self,
        id: str,
        *,
        org_id: str | None = None,
        status: Literal["PENDING", "APPROVED"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BillResponse:
        """
        Updates the status of a specified Bill with the given Bill ID.

        This endpoint allows you to transition a Bill's status through various stages,
        such as from "Pending" to "Approved".

        Args:
          status: The new status you want to assign to the Bill. Must be one "Pending" or
              "Approved".

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/organizations/{org_id}/bills/{id}/status",
            body=await async_maybe_transform({"status": status}, bill_update_status_params.BillUpdateStatusParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BillResponse,
        )


class BillsResourceWithRawResponse:
    def __init__(self, bills: BillsResource) -> None:
        self._bills = bills

        self.retrieve = to_raw_response_wrapper(
            bills.retrieve,
        )
        self.list = to_raw_response_wrapper(
            bills.list,
        )
        self.delete = to_raw_response_wrapper(
            bills.delete,
        )
        self.approve = to_raw_response_wrapper(
            bills.approve,
        )
        self.latest_by_account = to_raw_response_wrapper(
            bills.latest_by_account,
        )
        self.lock = to_raw_response_wrapper(
            bills.lock,
        )
        self.search = to_raw_response_wrapper(
            bills.search,
        )
        self.update_status = to_raw_response_wrapper(
            bills.update_status,
        )

    @cached_property
    def credit_line_items(self) -> CreditLineItemsResourceWithRawResponse:
        return CreditLineItemsResourceWithRawResponse(self._bills.credit_line_items)

    @cached_property
    def debit_line_items(self) -> DebitLineItemsResourceWithRawResponse:
        return DebitLineItemsResourceWithRawResponse(self._bills.debit_line_items)

    @cached_property
    def line_items(self) -> LineItemsResourceWithRawResponse:
        return LineItemsResourceWithRawResponse(self._bills.line_items)


class AsyncBillsResourceWithRawResponse:
    def __init__(self, bills: AsyncBillsResource) -> None:
        self._bills = bills

        self.retrieve = async_to_raw_response_wrapper(
            bills.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            bills.list,
        )
        self.delete = async_to_raw_response_wrapper(
            bills.delete,
        )
        self.approve = async_to_raw_response_wrapper(
            bills.approve,
        )
        self.latest_by_account = async_to_raw_response_wrapper(
            bills.latest_by_account,
        )
        self.lock = async_to_raw_response_wrapper(
            bills.lock,
        )
        self.search = async_to_raw_response_wrapper(
            bills.search,
        )
        self.update_status = async_to_raw_response_wrapper(
            bills.update_status,
        )

    @cached_property
    def credit_line_items(self) -> AsyncCreditLineItemsResourceWithRawResponse:
        return AsyncCreditLineItemsResourceWithRawResponse(self._bills.credit_line_items)

    @cached_property
    def debit_line_items(self) -> AsyncDebitLineItemsResourceWithRawResponse:
        return AsyncDebitLineItemsResourceWithRawResponse(self._bills.debit_line_items)

    @cached_property
    def line_items(self) -> AsyncLineItemsResourceWithRawResponse:
        return AsyncLineItemsResourceWithRawResponse(self._bills.line_items)


class BillsResourceWithStreamingResponse:
    def __init__(self, bills: BillsResource) -> None:
        self._bills = bills

        self.retrieve = to_streamed_response_wrapper(
            bills.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            bills.list,
        )
        self.delete = to_streamed_response_wrapper(
            bills.delete,
        )
        self.approve = to_streamed_response_wrapper(
            bills.approve,
        )
        self.latest_by_account = to_streamed_response_wrapper(
            bills.latest_by_account,
        )
        self.lock = to_streamed_response_wrapper(
            bills.lock,
        )
        self.search = to_streamed_response_wrapper(
            bills.search,
        )
        self.update_status = to_streamed_response_wrapper(
            bills.update_status,
        )

    @cached_property
    def credit_line_items(self) -> CreditLineItemsResourceWithStreamingResponse:
        return CreditLineItemsResourceWithStreamingResponse(self._bills.credit_line_items)

    @cached_property
    def debit_line_items(self) -> DebitLineItemsResourceWithStreamingResponse:
        return DebitLineItemsResourceWithStreamingResponse(self._bills.debit_line_items)

    @cached_property
    def line_items(self) -> LineItemsResourceWithStreamingResponse:
        return LineItemsResourceWithStreamingResponse(self._bills.line_items)


class AsyncBillsResourceWithStreamingResponse:
    def __init__(self, bills: AsyncBillsResource) -> None:
        self._bills = bills

        self.retrieve = async_to_streamed_response_wrapper(
            bills.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            bills.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            bills.delete,
        )
        self.approve = async_to_streamed_response_wrapper(
            bills.approve,
        )
        self.latest_by_account = async_to_streamed_response_wrapper(
            bills.latest_by_account,
        )
        self.lock = async_to_streamed_response_wrapper(
            bills.lock,
        )
        self.search = async_to_streamed_response_wrapper(
            bills.search,
        )
        self.update_status = async_to_streamed_response_wrapper(
            bills.update_status,
        )

    @cached_property
    def credit_line_items(self) -> AsyncCreditLineItemsResourceWithStreamingResponse:
        return AsyncCreditLineItemsResourceWithStreamingResponse(self._bills.credit_line_items)

    @cached_property
    def debit_line_items(self) -> AsyncDebitLineItemsResourceWithStreamingResponse:
        return AsyncDebitLineItemsResourceWithStreamingResponse(self._bills.debit_line_items)

    @cached_property
    def line_items(self) -> AsyncLineItemsResourceWithStreamingResponse:
        return AsyncLineItemsResourceWithStreamingResponse(self._bills.line_items)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["Address"]


class Address(BaseModel):
    address_line1: Optional[str] = FieldInfo(alias="addressLine1", default=None)

    address_line2: Optional[str] = FieldInfo(alias="addressLine2", default=None)

    address_line3: Optional[str] = FieldInfo(alias="addressLine3", default=None)

    address_line4: Optional[str] = FieldInfo(alias="addressLine4", default=None)

    country: Optional[str] = None

    locality: Optional[str] = None

    post_code: Optional[str] = FieldInfo(alias="postCode", default=None)

    region: Optional[str] = None
